//#include "metalShaderTypes.h"

typedef struct
{
	float4 srcColor;
	float tolerance;
} fxVars;


vertex fxVertexOut vertexFunc(uint vertexID [[ vertex_id ]]
							  ,const device fxShaderVerts* in[[ buffer(0) ]]
							  ,texture2d<half> inputTex0 [[ texture(0) ]]
							  ,texture2d<half> inputTexMask [[ texture(1) ]]
							  ,constant fxGeneralUniforms& u[[ buffer(1) ]]
							  ,constant fxVars& vars[[ buffer(2)]])
{
    fxVertexOut out;

	out.normPos=in[vertexID].pos.xy*vec2(.5,.5f)+vec2(.5,.5);
	out.position=in[vertexID].pos;

    out.tex1=in[vertexID].tex1;
	out.texMask=in[vertexID].texMask;
	out.texSize=float2(inputTex0.get_width(),inputTex0.get_height());
    return out;
}

fragment float4 fragmentFunc(fxVertexOut in [[stage_in]]
							 ,texture2d<half> inputTex [[ texture(0) ]]
							 ,texture2d<half> inputTexMask [[ texture(1) ]]
							 ,constant fxGeneralUniforms& u[[ buffer(0) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
    vec4 texColor = tex(inputTex, fsTexture);
	
	float4 outColor=texColor;
	vec3 diff=abs(texColor.rgb-vars.srcColor.rgb);
	if(diff.r<vars.tolerance)
	{
		if(diff.g<vars.tolerance)
		{
			if(diff.b<vars.tolerance)
			{
				outColor.a=1.0-u.blendValue;
			}
		}
	}

	float blendValue=u.blendValue;
#ifdef USE_MASK
	blendValue*=tex(inputTexMask,in.texMask).a;
#endif
    outColor.rgb=mix(texColor.rgb,outColor.rgb,blendValue);
	return outColor;
}
