#version 330

uniform sampler2DRect inputTex0;
uniform sampler2DRect inputTex1;
uniform sampler2DRect inputTex2;
uniform sampler2DRect inputTex3;
uniform sampler2DRect inputTexMask;
uniform vec4 endColor;
uniform float endOpacity;

const vec4 std709R = vec4( 1.793,1.164383, 0.000000,0.0  );
const vec4 std709G = vec4(-0.534,1.164383,-0.213,0.0  );
const vec4 std709B = vec4( 0.000000,1.164383, 2.115,0.0  );
const vec4 stdbias = vec4(-0.5  , -0.0625,  -0.5   ,0.0  );
const vec4 scale10to16 = vec4(64.0, 64.0, 64.0, 64.0);
const vec4 scale12to16 = vec4(16.0, 16.0, 16.0, 16.0);


in vec2 fsTexY;
in vec2 fsTexUV;
in vec2 fsTextureMask;
out vec4 outColor;

void main()
{
    vec4 yuvIn;
    vec4 yuvOut;

    yuvIn.g = texture(inputTex0, fsTexY).r;
    yuvIn.b = texture(inputTex1, fsTexUV).r;
    yuvIn.r = texture(inputTex2, fsTexUV).r;
    
#ifdef HAS_ALPHA
    yuvIn.a = texture(inputTex3, fsTexY).r;
#endif

#ifdef IS_10BIT
    yuvIn*=scale10to16;
#endif
    
#ifdef IS_12BIT
    yuvIn*=scale12to16;
#endif
    
    yuvIn+=stdbias;
	
#ifndef HAS_ALPHA
    yuvIn.a=1.0;
#endif
    
    outColor=vec4(dot(std709R, yuvIn),
                  dot(std709G, yuvIn),
                  dot(std709B, yuvIn),
                  yuvIn.a);

#ifndef NO_PREMULT
    outColor.rgb*=outColor.a;
#endif

#ifdef USE_MASK
    outColor.a*=texture(inputTexMask,fsTextureMask).a;
#endif

#ifdef USE_END_COLOR
    outColor=mix(endColor,outColor,endOpacity);
#endif

}
